/* 
 * Gnome Memory Blocks
 *
 * Copyright (C) 2000 The Free Software Foundation
 *
 * Author: Vova Babin <vb@dwuj.ichf.edu.pl>
 */

/*
 *   This code is not good. So it will be enhanced (i hope) in next realise
 */

#include <gmodule.h>
#include "game-item-animation.h"
#include "scaling-frames.h"	/* scaling_frame1 and scaling_frame2 equal size images */

#define V_BORDER 8
#define H_BORDER 8
#define FRAME_WIDTH scaling_frame1.width
#define FRAME_HEIGHT scaling_frame1.height
#define PICTURE_WIDTH (FRAME_WIDTH - 2*H_BORDER)
#define PICTURE_HEIGHT (FRAME_HEIGHT - 2*V_BORDER)
#define ANIMATION_TIMEOUT 50

static GdkPixbuf *pixbuf1 = NULL;
static GdkPixbuf *pixbuf2 = NULL;

G_MODULE_EXPORT void
g_module_unload (GModule * module)
{
  if (pixbuf1)
    {
      gdk_pixbuf_unref (pixbuf1);
      pixbuf1 = NULL;
    }
  if (pixbuf2)
    {
      gdk_pixbuf_unref (pixbuf2);
      pixbuf2 = NULL;
    }
}

G_MODULE_EXPORT const gchar *
g_module_check_init (GModule * module)
{
  pixbuf1 = gdk_pixbuf_new_from_data (scaling_frame1.pixel_data,
				      GDK_COLORSPACE_RGB,
				      TRUE,
				      8,
				      FRAME_WIDTH,
				      FRAME_HEIGHT,
				      4 * FRAME_WIDTH, NULL, NULL);
  if (!pixbuf1)
    {
      return "failed to create tile pictures";
    }

  pixbuf2 = gdk_pixbuf_new_from_data (scaling_frame2.pixel_data,
				      GDK_COLORSPACE_RGB,
				      TRUE,
				      8,
				      FRAME_WIDTH,
				      FRAME_HEIGHT,
				      4 * FRAME_WIDTH, NULL, NULL);
  if (!pixbuf2)
    {
      gdk_pixbuf_unref (pixbuf1);
      return "failed to create tile pictures";
    }
  return NULL;
}

GameItemAnimation *
create_flip (GdkPixbuf * picture)
{
  GdkPixbuf *composited, *tmp, *pic_sized;
  GameItemAnimation *flip;
  GameItemFrame *frame;
  guint new_w;
  gfloat done;

  flip = game_item_animation_new ();
  if (gdk_pixbuf_get_width (picture) != PICTURE_WIDTH ||
      gdk_pixbuf_get_height (picture) != PICTURE_HEIGHT)
    pic_sized =
      gdk_pixbuf_scale_simple (picture, PICTURE_WIDTH, PICTURE_HEIGHT,
			       GDK_INTERP_HYPER);
  else
    {
      pic_sized = picture;
      gdk_pixbuf_ref (pic_sized);
    }

  frame = game_item_frame_new (pixbuf1, 0, 0, ANIMATION_TIMEOUT / 2);
  game_item_animation_append_frame (flip, frame);

  done = 0.1;
  while (done < 1.0)
    {
      new_w = (guint) (((gfloat) FRAME_WIDTH) * (1.0 - done));
      tmp =
	gdk_pixbuf_scale_simple (pixbuf1, new_w, FRAME_HEIGHT,
				 GDK_INTERP_NEAREST);
      frame =
	game_item_frame_new (tmp, (FRAME_WIDTH - new_w) / 2, 0,
			     ANIMATION_TIMEOUT);
      gdk_pixbuf_unref (tmp);
      game_item_animation_append_frame (flip, frame);
      done += 0.3;
    }
  composited = gdk_pixbuf_copy (pixbuf2);
  gdk_pixbuf_composite (pic_sized, composited, 0, 0, FRAME_WIDTH,
			FRAME_HEIGHT, (gdouble) H_BORDER, (gdouble) V_BORDER,
			1.0, 1.0, GDK_INTERP_HYPER, 255);
  done = 0.1;
  while (done < 1.0)
    {
      new_w = (guint) (((gfloat) PICTURE_WIDTH) * done);
      tmp =
	gdk_pixbuf_scale_simple (composited, new_w, FRAME_HEIGHT,
				 GDK_INTERP_NEAREST);
      frame =
	game_item_frame_new (tmp, (FRAME_WIDTH - new_w) / 2, 0,
			     ANIMATION_TIMEOUT);
      gdk_pixbuf_unref (tmp);
      game_item_animation_append_frame (flip, frame);
      done += 0.3;
    }
  frame = game_item_frame_new (composited, 0, 0, ANIMATION_TIMEOUT);
  gdk_pixbuf_unref (composited);
  game_item_animation_append_frame (flip, frame);
  gdk_pixbuf_unref (pic_sized);
  return flip;
}

GameItemAnimation *
create_hide (GdkPixbuf * picture)
{
  GdkPixbuf *composited, *tmp, *pic_sized;
  GameItemAnimation *hide;
  GameItemFrame *frame;
  guint new_w, new_h;
  gfloat done;

  hide = game_item_animation_new ();
  if (gdk_pixbuf_get_width (picture) != PICTURE_WIDTH ||
      gdk_pixbuf_get_height (picture) != PICTURE_HEIGHT)
    pic_sized =
      gdk_pixbuf_scale_simple (picture, PICTURE_WIDTH, PICTURE_HEIGHT,
			       GDK_INTERP_HYPER);
  else
    {
      pic_sized = picture;
      gdk_pixbuf_ref (pic_sized);
    }
  composited = gdk_pixbuf_copy (pixbuf2);
  gdk_pixbuf_composite (picture, composited, 0, 0, FRAME_WIDTH, FRAME_HEIGHT,
			(gdouble) H_BORDER, (gdouble) V_BORDER, 1.0, 1.0,
			GDK_INTERP_HYPER, 255);
  done = 0.9;
  while (done > 0.0)
    {
      new_w = (guint) (((gfloat) PICTURE_WIDTH) * done);
      new_h = (guint) (((gfloat) PICTURE_HEIGHT) * done);
      tmp =
	gdk_pixbuf_scale_simple (composited, new_w, new_h,
				 GDK_INTERP_NEAREST);
      frame =
	game_item_frame_new (tmp, (FRAME_WIDTH - new_w) / 2,
			     (FRAME_HEIGHT - new_h) / 2, ANIMATION_TIMEOUT);
      gdk_pixbuf_unref (tmp);
      game_item_animation_append_frame (hide, frame);
      done -= 0.2;
    }
  gdk_pixbuf_unref (composited);
  gdk_pixbuf_unref (pic_sized);
  return hide;
}
