/*
 * gtk-clock: The GTK clock widget
 * (C)1998 The Free Software Foundation
 *
 * Author: Szekeres Istvn (szekeres@cyberspace.mht.bme.hu)
 */

/*
 * Slightly modified (modifications not finished yet)
 * by Svyatoslav Kondrat <valiska@lulu.ichf.edu.pl>
 * for Gnome Memory Blocks
 * 30.08.2000
 */

#include <math.h>
#include "game-clock.h"

static void game_clock_class_init (GameClockClass * klass);
static void game_clock_init (GameClock * clock);

static GtkToolbarClass *parent_class = NULL;

GtkType game_clock_get_type (void)
{
  static GtkType game_clock_type = 0;
  if (!game_clock_type)
    {
      static const GtkTypeInfo game_clock_info = {
	"GameClock",
	sizeof (GameClock),
	sizeof (GameClockClass),
	(GtkClassInitFunc) game_clock_class_init,
	(GtkObjectInitFunc) game_clock_init,
	NULL, NULL,
	(GtkClassInitFunc) NULL
      };
      game_clock_type =
	gtk_type_unique (gtk_toolbar_get_type (), &game_clock_info);
    }
  return game_clock_type;
}


static void
game_clock_orientation_changed (GameClock * clock,
				GtkOrientation orientation, gpointer data)
{
  gint i;

  g_return_if_fail (clock != NULL);
  g_return_if_fail (GTK_IS_GAME_CLOCK (clock));

  for (i = 0; i < 5; i++)
    game_indicator_set_orientation (GAME_INDICATOR (clock->digits[i]),
				    orientation);
}


static void
game_clock_finalize (GtkObject * object)
{
  GameClock *clock;

  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_GAME_CLOCK (object));

  clock = (GameClock *) object;
  if (clock->timer_id)
    gtk_timeout_remove (clock->timer_id);

  g_free (clock->tm);
  GTK_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
game_clock_class_init (GameClockClass * klass)
{
  GtkObjectClass *object_class = (GtkObjectClass *) klass;

  object_class->finalize = game_clock_finalize;
  parent_class = gtk_type_class (gtk_toolbar_get_type ());
}

static void
game_clock_init (GameClock * clock)
{
  gint i;

  clock->timer_id = 0;
  clock->update_interval = 1;
  clock->seconds = time (NULL);
  clock->stopped = 0;

  clock->direction = GAME_DIGIT_DIRECTION_NORMAL;

  gtk_container_set_border_width (GTK_CONTAINER (clock), 4);

  for (i = 0; i < 5; i++)
    {
      clock->digits[i] = (GameDigit *) game_digit_new_default ();
      gtk_toolbar_append_widget (GTK_TOOLBAR (clock),
				 GTK_WIDGET (clock->digits[i]), NULL, NULL);
      gtk_widget_show (GTK_WIDGET (clock->digits[i]));
    }

  game_digit_set_semicolons (clock->digits[2]);
  gtk_signal_connect (GTK_OBJECT (clock), "orientation_changed",
		      GTK_SIGNAL_FUNC (game_clock_orientation_changed), NULL);
}

static void
game_clock_gen_digits (GameClock * clock)
{
  GtkToolbar *toolbar;
  time_t secs;

  g_return_if_fail (clock != NULL);
  g_return_if_fail (GTK_IS_GAME_CLOCK (clock));

  toolbar = (GtkToolbar *) clock;

  switch (clock->type)
    {
    case GAME_CLOCK_DECREASING:
      secs = clock->seconds - time (NULL);
      g_message
	(" Decreasing clock is not implemented so far; do not work properly ...\n");
      break;
    case GAME_CLOCK_INCREASING:
      if (clock->timer_id)
	secs = time (NULL) - clock->seconds;
      else
	secs = clock->stopped;
      break;
    case GAME_CLOCK_REALTIME:
      g_message
	(" Real-Time clock is not implemented so far; do not work properly ...\n");
      secs = time (NULL);
      break;
    default:
      g_assert_not_reached ();
    }

  if (clock->type == GAME_CLOCK_REALTIME)
    clock->tm = localtime (&secs);
  else
    {
      clock->tm->tm_hour = secs / 3600;
      secs -= clock->tm->tm_hour * 3600;
      clock->tm->tm_min = secs / 60;
      clock->tm->tm_sec = secs - clock->tm->tm_min * 60;
    }

  if (game_digit_is_semicolons (clock->digits[2]) && clock->timer_id)
    game_digit_set_empty (clock->digits[2]);
  else
    game_digit_set_semicolons (clock->digits[2]);

  if ((clock->direction == GAME_DIGIT_DIRECTION_NORMAL &&
       toolbar->orientation == GTK_ORIENTATION_HORIZONTAL) ||
      (clock->direction == GAME_DIGIT_DIRECTION_INVERSE &&
       toolbar->orientation == GTK_ORIENTATION_VERTICAL))
    {
      game_digit_set_digit (clock->digits[4], clock->tm->tm_sec % 10);
      game_digit_set_digit (clock->digits[3],
			    (guint) floor (clock->tm->tm_sec / 10));
      game_digit_set_digit (clock->digits[1], clock->tm->tm_min % 10);
      game_digit_set_digit (clock->digits[0],
			    (guint) floor (clock->tm->tm_min / 10));
    }
  else
    {
      game_digit_set_digit (clock->digits[0], clock->tm->tm_sec % 10);
      game_digit_set_digit (clock->digits[1],
			    (guint) floor (clock->tm->tm_sec / 10));
      game_digit_set_digit (clock->digits[3], clock->tm->tm_min % 10);
      game_digit_set_digit (clock->digits[4],
			    (guint) floor (clock->tm->tm_min / 10));
    }
}

static gboolean
game_clock_timer_callback (gpointer data)
{
  GameClock *clock = (GameClock *) data;

  GDK_THREADS_ENTER ();
  game_clock_gen_digits (clock);
  GDK_THREADS_LEAVE ();

  return TRUE;
}

static gboolean
game_clock_timer_first_callback (gpointer data)
{
  GameClock *clock = (GameClock *) data;
  guint tmpid;

  GDK_THREADS_ENTER ();
  game_clock_gen_digits (clock);

  tmpid =
    gtk_timeout_add (1000 * clock->update_interval, game_clock_timer_callback,
		     clock);
  game_clock_stop (clock);
  clock->timer_id = tmpid;
  GDK_THREADS_LEAVE ();
  return FALSE;
}

/*
 *  Public functions
 */

GtkWidget *
game_clock_new (GameClockType type)
{
  GameClock *clock = gtk_type_new (game_clock_get_type ());

  clock->type = type;

  if (type == GAME_CLOCK_REALTIME)
    {
      clock->update_interval = 60;
      clock->tm = localtime (&clock->seconds);
      clock->timer_id = gtk_timeout_add (1000 * (60 - clock->tm->tm_sec),
					 game_clock_timer_first_callback,
					 clock);
    }
  else
    {
      clock->tm = g_new0 (struct tm, 1);
      clock->update_interval = 1;
    }

  game_clock_gen_digits (clock);
  return GTK_WIDGET (clock);
}

void
game_clock_set_seconds (GameClock * clock, time_t seconds)
{
  g_return_if_fail (clock != NULL);

  if (clock->type == GAME_CLOCK_INCREASING)
    clock->seconds = time (NULL) - seconds;
  else if (clock->type == GAME_CLOCK_DECREASING)
    clock->seconds = time (NULL) + seconds;

  if (!clock->timer_id)
    clock->stopped = seconds;

  game_clock_gen_digits (clock);
}

void
game_clock_set_update_interval (GameClock * clock, gint seconds)
{
  gint tmp;

  g_return_if_fail (GTK_IS_GAME_CLOCK (clock));

  tmp = clock->update_interval;
  clock->update_interval = seconds;

  if (tmp > seconds && clock->timer_id)
    {
      game_clock_stop (clock);
      game_clock_start (clock);
    }
}

void
game_clock_start (GameClock * clock)
{
  g_return_if_fail (GTK_IS_GAME_CLOCK (clock));

  if (clock->timer_id)
    {
      g_warning ("attempt to start already started clock");
      return;
    }

  game_clock_set_seconds (clock, 0);
  clock->timer_id = gtk_timeout_add (1000 * clock->update_interval,
				     game_clock_timer_callback, clock);
}

void
game_clock_continue (GameClock * clock)
{
  g_return_if_fail (GTK_IS_GAME_CLOCK (clock));

  if (clock->timer_id)
    return;

  game_clock_set_seconds (clock, clock->stopped);
  clock->timer_id = gtk_timeout_add (1000 * clock->update_interval,
				     game_clock_timer_callback, clock);
}

void
game_clock_stop (GameClock * clock)
{
  g_return_if_fail (GTK_IS_GAME_CLOCK (clock));

  if (!clock->timer_id)
    return;

  if (clock->type == GAME_CLOCK_INCREASING)
    clock->stopped = time (NULL) - clock->seconds;
  else if (clock->type == GAME_CLOCK_DECREASING)
    clock->stopped = clock->seconds - time (NULL);

  gtk_timeout_remove (clock->timer_id);
  clock->timer_id = 0;
  game_clock_gen_digits (clock);
}

void
game_clock_set_direction (GameClock * clock, GameDigitDirection direction)
{
  gint i;

  g_return_if_fail (GTK_IS_GAME_CLOCK (clock));

  clock->direction = direction;
  for (i = 0; i < 5; i++)
    game_digit_set_direction (clock->digits[i], direction);

  game_clock_gen_digits (clock);
}


GameDigitDirection game_clock_get_direction (GameClock * clock)
{
  g_return_val_if_fail (GTK_IS_GAME_CLOCK (clock), 0);

  return clock->direction;
}
