/*
 * Gnome Memory Blocks
 * (C) 2000 the Free Software Foundation
 *
 * Author: Svyatoslav Kondrat <valiska@lulu.ichf.edu.pl>
 */

#ifndef __GAME_DIGIT_H__
#define __GAME_DIGIT_H__

#ifdef __cplusplus
extern "C"
{
#endif				/* __cplusplus */

#include <gdk/gdk.h>
#include <gtk/gtk.h>

/*
 *   Game Indicator (base class for GameDigit)
 */

#define GTK_TYPE_GAME_INDICATOR            (game_indicator_get_type ())
#define GAME_INDICATOR(obj)                (GTK_CHECK_CAST ((obj), GTK_TYPE_GAME_INDICATOR, GameIndicator))
#define GAME_INDICATOR_CLASS(klass)        (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_GAME_INDICATOR, GameIndicatorClass))
#define GTK_IS_GAME_INDICATOR(obj)         (GTK_CHECK_TYPE ((obj), GTK_TYPE_GAME_INDICATOR))
#define GTK_IS_GAME_INDICATOR_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_GAME_INDICATOR))

  typedef enum
  {
    GAME_DIGIT_DIRECTION_NORMAL,
    GAME_DIGIT_DIRECTION_INVERSE
  }
  GameDigitDirection;

  typedef enum
  {
    GAME_INDICATOR_EMPTY = 0,
    GAME_INDICATOR_A = 1 << 0,
    GAME_INDICATOR_B = 1 << 1,
    GAME_INDICATOR_C = 1 << 2,
    GAME_INDICATOR_D = 1 << 3,
    GAME_INDICATOR_E = 1 << 4,
    GAME_INDICATOR_F = 1 << 5,
    GAME_INDICATOR_G = 1 << 6,
    GAME_INDICATOR_H = 1 << 7,
    GAME_INDICATOR_O = 1 << 8
  }
  GameIndicatorBars;

  typedef struct _GameIndicator GameIndicator;
  typedef struct _GameIndicatorClass GameIndicatorClass;

  GtkType game_indicator_get_type (void);
  GtkWidget *game_indicator_new (GtkOrientation);
  GtkWidget *game_indicator_new_with_bars (GtkOrientation, GameIndicatorBars);
  void game_indicator_set_orientation (GameIndicator *, GtkOrientation);
  void game_indicator_set_bars (GameIndicator *, GameIndicatorBars);
  GameIndicatorBars game_indicator_get_bars (GameIndicator *);

/*
 *   Game Digit
 */

#define GTK_TYPE_GAME_DIGIT                (game_digit_get_type ())
#define GAME_DIGIT(obj)                    (GTK_CHECK_CAST ((obj), GTK_TYPE_GAME_DIGIT, GameDigit))
#define GAME_DIGIT_CLASS(klass)            (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_GAME_DIGIT, GameDigitClass))
#define GTK_IS_GAME_DIGIT(obj)             (GTK_CHECK_TYPE ((obj), GTK_TYPE_GAME_DIGIT))
#define GTK_IS_GAME_DIGIT_CLASS(klass)     (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_GAME_DIGIT))

  typedef struct _GameDigit GameDigit;
  typedef struct _GameDigitClass GameDigitClass;

  GtkType game_digit_get_type (void);
  GtkWidget *game_digit_new_default (void);
  GtkWidget *game_digit_new (GtkOrientation, GameDigitDirection);
  GtkWidget *game_digit_new_with_digit (GtkOrientation, GameDigitDirection,
					guint digit);

  void game_digit_set_direction (GameDigit *, GameDigitDirection);
  GameDigitDirection game_digit_get_direction (GameDigit *);

  void game_digit_set_digit (GameDigit *, guint digit);
  guint game_digit_get_digit (GameDigit *);

  void game_digit_set_semicolons (GameDigit *);
  gboolean game_digit_is_semicolons (GameDigit *);

  void game_digit_set_empty (GameDigit *);
  gboolean game_digit_is_empty (GameDigit *);

#ifdef __cplusplus
}
#endif				/* __cplusplus */
#endif				/* __GAME_DIGIT_H__ */
