/* 
 * Gnome Memory Blocks
 *
 * Copyright (C) 2000 The Free Software Foundation
 *
 * Author: Vova Babin <vb@dwuj.ichf.edu.pl>
 */

#ifndef __GAME_ITEM_ANIMATION_H__
#define __GAME_ITEM_ANIMATION_H__

#include <gtk/gtkobject.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

/*
 *  We need those objects, becouse GdkPixbufAnimation becomes opaque
 */

#define GAME_ITEM_FRAME_TYPE            (game_item_frame_get_type ())
#define GAME_ITEM_FRAME(obj)            (GTK_CHECK_CAST (obj, GAME_ITEM_FRAME_TYPE, GameItemFrame))
#define GAME_ITEM_FRAME_CLASS(klass)    (GTK_CHECK_CLASS_CAST (klass, GAME_ITEM_FRAME_TYPE, GameItemFrameClass))
#define IS_GAME_ITEM_FRAME(obj)         (GTK_CHECK_TYPE ((obj), GAME_ITEM_FRAME_TYPE))
#define IS_GAME_ITEM_FRAME_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GAME_ITEM_FRAME_TYPE))

typedef struct _GameItemFrame GameItemFrame;
typedef struct _GameItemFrameClass GameItemFrameClass;

struct _GameItemFrame
{
  GtkObject object;
  /* The pixbuf with this frame's image data */
  GdkPixbuf *pixbuf;
  /* Offsets for overlaying onto the animation's area */
  guint x_offset;
  guint y_offset;
  /* Frame duration in ms */
  guint delay_time;
};

struct _GameItemFrameClass
{
  GtkObjectClass klass;
};

GtkType game_item_frame_get_type (void);
GameItemFrame *game_item_frame_new (GdkPixbuf * pixbuf,
				    guint x_offset,
				    guint y_offset, guint delay_time);

#define GAME_ITEM_ANIMATION_TYPE            (game_item_animation_get_type ())
#define GAME_ITEM_ANIMATION(obj)            (GTK_CHECK_CAST (obj, GAME_ITEM_ANIMATION_TYPE, GameItemAnimation))
#define GAME_ITEM_ANIMATION_CLASS(klass)    (GTK_CHECK_CLASS_CAST (klass, GAME_ITEM_ANIMATION_TYPE, GameItemAnimationClass))
#define IS_GAME_ITEM_ANIMATION(obj)         (GTK_CHECK_TYPE ((obj), GAME_ITEM_ANIMATION_TYPE))
#define IS_GAME_ITEM_ANIMATION_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GAME_ITEM_ANIMATION_TYPE))

typedef struct _GameItemAnimation GameItemAnimation;
typedef struct _GameItemAnimationClass GameItemAnimationClass;

GtkType game_item_animation_get_type (void);
GameItemAnimation *game_item_animation_new (void);
void game_item_animation_append_frame (GameItemAnimation *, GameItemFrame *);

guint game_item_animation_get_width (GameItemAnimation *);
guint game_item_animation_get_height (GameItemAnimation *);
guint game_item_animation_get_n_frames (GameItemAnimation *);
GameItemFrame *game_item_animation_get_nth_frame (GameItemAnimation *, guint);

#endif	/* __GAME_ITEM_ANIMATION_H__ */
