/* 
 * Gnome Memory Blocks
 *
 * Copyright (C) 2000 The Free Software Foundation
 *
 * Author: Vova Babin <vb@dwuj.ichf.edu.pl>
 */

#include <gmodule.h>
#include "game-item-generic-style.h"

static GameItemStyleClass *parent_class = NULL;

struct _GameItemGenericStyle
{
  GameItemStyle style;
  GModule *engine;
  GameItemAnimation *(*create_flip) (GdkPixbuf *);
  GameItemAnimation *(*create_hide) (GdkPixbuf *);
};

static void
game_item_generic_style_finalize (GtkObject * obj)
{
  GameItemGenericStyle *gen_style;

  g_return_if_fail (obj != NULL);
  g_return_if_fail (IS_GAME_ITEM_GENERIC_STYLE (obj));

  gen_style = GAME_ITEM_GENERIC_STYLE (obj);

  if (gen_style->engine)
    g_module_close (gen_style->engine);

  if (GTK_OBJECT_CLASS (parent_class)->finalize)
    GTK_OBJECT_CLASS (parent_class)->finalize (obj);
}

static GameItemAnimation *
game_item_generic_style_create_flip (GameItemStyle * style,
				     GdkPixbuf * picture)
{
  GameItemGenericStyle *gen_style;

  g_return_val_if_fail (IS_GAME_ITEM_GENERIC_STYLE (style), NULL);

  gen_style = GAME_ITEM_GENERIC_STYLE (style);
  if (gen_style->create_flip)
    return gen_style->create_flip (picture);
  else
    return NULL;
}

static GameItemAnimation *
game_item_generic_style_create_hide (GameItemStyle * style,
				     GdkPixbuf * picture)
{
  GameItemGenericStyle *gen_style;

  g_return_val_if_fail (IS_GAME_ITEM_GENERIC_STYLE (style), NULL);

  gen_style = GAME_ITEM_GENERIC_STYLE (style);
  if (gen_style->create_hide)
    return gen_style->create_hide (picture);
  else
    return NULL;
}

static void
game_item_generic_style_init (GameItemGenericStyle * style)
{
  g_return_if_fail (style != NULL);
  g_return_if_fail (IS_GAME_ITEM_GENERIC_STYLE (style));

  style->engine = NULL;
  style->create_flip = NULL;
  style->create_hide = NULL;
}

static void
game_item_generic_style_class_init (GameItemGenericStyleClass * klass)
{
  GtkObjectClass *object_class;
  GameItemStyleClass *style_class;

  g_return_if_fail (klass != NULL);
  g_return_if_fail (IS_GAME_ITEM_GENERIC_STYLE_CLASS (klass));

  parent_class = gtk_type_class (GAME_ITEM_STYLE_TYPE);

  object_class = (GtkObjectClass *) klass;
  object_class->finalize = game_item_generic_style_finalize;

  style_class = (GameItemStyleClass *) klass;
  style_class->create_flip_animation = game_item_generic_style_create_flip;
  style_class->create_hide_animation = game_item_generic_style_create_hide;
}

GtkType game_item_generic_style_get_type (void)
{
  static GtkType type = 0;
  if (!type)
    {
      static const GtkTypeInfo type_info = {
	"GameItemGenericStyle",
	sizeof (GameItemGenericStyle),
	sizeof (GameItemGenericStyleClass),
	(GtkClassInitFunc) game_item_generic_style_class_init,
	(GtkObjectInitFunc) game_item_generic_style_init,
	NULL, NULL,
	(GtkClassInitFunc) NULL
      };
      type = gtk_type_unique (GAME_ITEM_STYLE_TYPE, &type_info);
    }
  return type;
}

GameItemStyle *
game_item_generic_style_new (const gchar * filename)
{
  GameItemGenericStyle *style;
  GModule *engine = NULL;

  g_return_val_if_fail (filename != NULL, NULL);

  engine = g_module_open (filename, 0);
  if (!engine)
    {
      g_warning ("failed to load engine %s", filename);
      return NULL;
    }
  style =
    GAME_ITEM_GENERIC_STYLE (gtk_type_new (GAME_ITEM_GENERIC_STYLE_TYPE));
  if (!g_module_symbol
      (engine, "create_flip", (gpointer *) & (style->create_flip)))
    {
      g_warning (g_module_error ());
      g_module_close (engine);
      gtk_object_unref (GTK_OBJECT (style));
      return NULL;
    }

  style->engine = engine;
  g_module_symbol (engine, "create_hide",
		   (gpointer *) & (style->create_hide));
  return GAME_ITEM_STYLE (style);
}

const gchar *
game_item_generic_style_get_engine_name (GameItemGenericStyle * style)
{
  g_return_val_if_fail (style != NULL, NULL);
  g_return_val_if_fail (IS_GAME_ITEM_GENERIC_STYLE (style), NULL);

  return g_module_name (style->engine);
}
