/* 
 * Gnome Memory Blocks
 *
 * Copyright (C) 2000 The Free Software Foundation
 *
 * Author: Vova Babin <vb@dwuj.ichf.edu.pl>
 */

#ifndef __GAME_ITEM_STYLE_H__
#define __GAME_ITEM_STYLE_H__

#include "game-item-animation.h"

#define GAME_ITEM_STYLE_TYPE            (game_item_style_get_type ())
#define GAME_ITEM_STYLE(obj)            (GTK_CHECK_CAST (obj, GAME_ITEM_STYLE_TYPE, GameItemStyle))
#define GAME_ITEM_STYLE_CLASS(klass)    (GTK_CHECK_CLASS_CAST (klass, GAME_ITEM_STYLE_TYPE, GameItemStyleClass))
#define IS_GAME_ITEM_STYLE(obj)         (GTK_CHECK_TYPE ((obj), GAME_ITEM_STYLE_TYPE))
#define IS_GAME_ITEM_STYLE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GAME_ITEM_STYLE_TYPE))

typedef struct _GameItemStyle GameItemStyle;
typedef struct _GameItemStyleClass GameItemStyleClass;

struct _GameItemStyle
{
  GtkObject object;
  /* privat */
  GHashTable *flip_hash;
  GHashTable *hide_hash;
};

struct _GameItemStyleClass
{
  GtkObjectClass klass;
  /* to be overriden by childs */
  GameItemAnimation *(*create_flip_animation) (GameItemStyle *, GdkPixbuf *);
  GameItemAnimation *(*create_hide_animation) (GameItemStyle *, GdkPixbuf *);
};

GtkType game_item_style_get_type (void);

/*
   the first frame of the flip animation
   correspond to GAME_ITEM_PASSIVE item state;
   and the last -- to the GAME_ITEM_ACTIVE,
   so it's must it must contain at least two frames

   hide animation can be empty e.g NULL (or n_frames = 0);
*/

GameItemAnimation *game_item_style_get_flip_animation (GameItemStyle *,
						       GdkPixbuf *);
GameItemAnimation *game_item_style_get_hide_animation (GameItemStyle *,
						       GdkPixbuf *);

#endif	/* __GAME_ITEM_STYLE_H__ */
