/* 
 * Gnome Memory Blocks
 *
 * Copyright (C) 2000 The Free Software Foundation
 *
 * Author: Vova Babin <vb@dwuj.ichf.edu.pl>
 */

#ifndef __GAME_ITEM_H__
#define __GAME_ITEM_H__

#include <gtk/gtkwidget.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "game-item-style.h"

#define GAME_ITEM_TYPE            (game_item_get_type ())
#define GAME_ITEM(obj)            (GTK_CHECK_CAST (obj, GAME_ITEM_TYPE, GameItem))
#define GAME_ITEM_CLASS(klass)    (GTK_CHECK_CLASS_CAST (klass, GAME_ITEM_TYPE, GameItemClass))
#define IS_GAME_ITEM(obj)         (GTK_CHECK_TYPE ((obj), GAME_ITEM_TYPE))
#define IS_GAME_ITEM_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GAME_ITEM_TYPE))

typedef struct _GameItem GameItem;
typedef struct _GameItemClass GameItemClass;

struct _GameItemClass
{
  GtkWidgetClass klass;
  void (*clicked) (GameItem *);
  void (*operation_finished) (GameItem *);
};

typedef enum
{
  GAME_ITEM_PASSIVE,
  GAME_ITEM_ACTIVE,
  GAME_ITEM_HIDDEN,
  GAME_ITEM_IN_FLIP_TO_ACTIVE,
  GAME_ITEM_IN_FLIP_TO_PASSIVE,
  GAME_ITEM_IN_HIDE,
  LAST_STATE			/* illegal one */
}
GameItemState;

/*
                   Item States Diagram
                   -------------------

              +--------------------+
      +--->---| IN_FLIP_TO_PASSIVE |---->---+
      |       +--------------------+        |
  +--------+                           +---------+
  | ACTIVE |                           | PASSIVE |----+
  +--------+                           +---------+    |
      |       +-------------------+         |         |
      +---<---| IN_FLIP_TO_ACTIVE |----<----+         |
              +-------------------+                   |
	                                              |
            +--------+       +---------+	      |
            | HIDDEN |---<---| IN_HIDE |-----<--------+
            +--------+       +---------+
*/

GtkType game_item_get_type (void);
GameItem *game_item_new (void);

void game_item_set_style (GameItem *, GameItemStyle *);
const GameItemStyle *game_item_get_style (const GameItem *);

void game_item_set_picture (GameItem *, GdkPixbuf *);
GdkPixbuf *game_item_get_picture (GameItem *);

GameItemState game_item_get_state (GameItem *);

void game_item_flip (GameItem *);
void game_item_hide (GameItem *);
void game_item_reset (GameItem *);

#endif	/* __GAME_ITEM_H__ */
